VERSION 5.00
Begin VB.UserControl Graph 
   BackColor       =   &H000000FF&
   ClientHeight    =   1620
   ClientLeft      =   0
   ClientTop       =   0
   ClientWidth     =   1905
   LockControls    =   -1  'True
   ScaleHeight     =   1620
   ScaleWidth      =   1905
   Begin VB.PictureBox Picture1 
      Appearance      =   0  'Flat
      BackColor       =   &H80000005&
      BorderStyle     =   0  'None
      ForeColor       =   &H80000008&
      Height          =   1095
      Left            =   150
      ScaleHeight     =   1095
      ScaleWidth      =   1305
      TabIndex        =   0
      Top             =   120
      Width           =   1305
   End
End
Attribute VB_Name = "Graph"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
'/////////////////////////////////////
'// general graph drawing control
'// 6th march 2001
'//
'// minimum error checking
'// capabilities of drawing and
'// connecting several different
'// graph lines and drawing
'// predefined grid lines
'//
'/////////////////////////////////////
'// properties
'//
'// Canvas
'// AutoRedraw
'// BackColor
'// GridColor
'// OriginColor
'// GraphColor
'//
'// GridStyle
'// OriginStyle
'// GraphStyle
'//
'// GridWidth
'// OriginWidth
'// GraphWidth
'//
'// XMin
'// XMax
'// YMin
'// YMax
'//
'/////////////////////////////////////
'// events
'//
'// Resize
'// MouseDown
'// MouseMove
'// MouseUp
'//
'/////////////////////////////////////
'// member functions
'//
'// Limit
'// Grid
'//
'// DrawDotLine
'// DrawGridLine
'//
'// Clear
'// Refresh
'// Start
'// Connect
'//
'/////////////////////////////////////

Option Explicit

'control properties
Dim grid_color As OLE_COLOR
Dim origin_color As OLE_COLOR
Dim grid_style As DrawStyleConstants
Dim origin_style As DrawStyleConstants
Dim grid_width As Long
Dim origin_width As Long

'drawing parameters
Dim min_x As Double
Dim max_x As Double
Dim min_y As Double
Dim max_y As Double

'distance between grid line
Dim scale_x As Double
Dim scale_y As Double
Dim div_x As Integer
Dim div_y As Integer

'whether graph drawing has already been started
Dim graph_start As Boolean

'public events
Public Event Resize()
Public Event Paint()
Public Event MouseDown(Button As Integer, Shift As Integer, x As Single, y As Single)
Public Event MouseMove(Button As Integer, Shift As Integer, x As Single, y As Single)
Public Event MouseUp(Button As Integer, Shift As Integer, x As Single, y As Single)

'/////////////////
'// interface

Private Sub UserControl_Initialize()

  'default drawing mode
  grid_color = vbCyan
  origin_color = vbBlack
  grid_style = vbDot
  origin_style = vbSolid
  grid_width = 1
  origin_width = 1

  Picture1.BackColor = vbWhite
  Picture1.ForeColor = vbBlue
  Picture1.DrawStyle = vbSolid
  Picture1.DrawWidth = 1

  'default scaling
  min_x = 0
  max_x = 100
  min_y = 0
  max_y = 100

  scale_x = 10
  scale_y = 10
  div_x = 2
  div_y = 2

  graph_start = True
  ScalingControl
End Sub

Private Sub UserControl_Resize()
  Picture1.Move 0, 0, Width, Height
End Sub

'read and save control properties
Private Sub UserControl_ReadProperties(PropBag As PropertyBag)
  AutoRedraw = PropBag.ReadProperty("AutoRedraw", AutoRedraw)
  BackColor = PropBag.ReadProperty("BackColor", BackColor)
  GridColor = PropBag.ReadProperty("GridColor", GridColor)
  OriginColor = PropBag.ReadProperty("OriginColor", OriginColor)
  GraphColor = PropBag.ReadProperty("GraphColor", GraphColor)
  GridStyle = PropBag.ReadProperty("GridStyle", GridStyle)
  OriginStyle = PropBag.ReadProperty("OriginStyle", OriginStyle)
  GraphStyle = PropBag.ReadProperty("GraphStyle", GraphStyle)
  GridWidth = PropBag.ReadProperty("GridWidth", GridWidth)
  OriginWidth = PropBag.ReadProperty("OriginWidth", OriginWidth)
  GraphWidth = PropBag.ReadProperty("GraphWidth", GraphWidth)
  DrawMode = PropBag.ReadProperty("DrawMode", DrawMode)
  XMin = PropBag.ReadProperty("XMin", XMin)
  XMax = PropBag.ReadProperty("XMax", XMax)
  YMin = PropBag.ReadProperty("YMin", YMin)
  YMax = PropBag.ReadProperty("YMax", YMax)
  XScale = PropBag.ReadProperty("XScale", XScale)
  YScale = PropBag.ReadProperty("YScale", YScale)
  XGrid = PropBag.ReadProperty("XGrid", XGrid)
  YGrid = PropBag.ReadProperty("YGrid", YGrid)
End Sub

Private Sub UserControl_WriteProperties(PropBag As PropertyBag)
  PropBag.WriteProperty "AutoRedraw", AutoRedraw, False
  PropBag.WriteProperty "BackColor", BackColor, vbBlack
  PropBag.WriteProperty "GridColor", GridColor, vbCyan
  PropBag.WriteProperty "OriginColor", OriginColor, vbBlack
  PropBag.WriteProperty "GraphColor", GraphColor, vbBlue
  PropBag.WriteProperty "GridStyle", GridStyle, vbDot
  PropBag.WriteProperty "OriginStyle", OriginStyle, vbSolid
  PropBag.WriteProperty "GraphStyle", GraphStyle, vbSolid
  PropBag.WriteProperty "GridWidth", GridWidth, 1
  PropBag.WriteProperty "OriginWidth", OriginWidth, 1
  PropBag.WriteProperty "GraphWidth", GraphWidth, 1
  PropBag.WriteProperty "DrawMode", DrawMode, 13
  PropBag.WriteProperty "XMin", XMin, 0
  PropBag.WriteProperty "XMax", XMax, 100
  PropBag.WriteProperty "YMin", YMin, 0
  PropBag.WriteProperty "YMax", YMax, 100
  PropBag.WriteProperty "XScale", XScale, 10
  PropBag.WriteProperty "YScale", YScale, 10
  PropBag.WriteProperty "XGrid", XGrid, 2
  PropBag.WriteProperty "YGrid", YGrid, 2
End Sub

'//////////////////
'// properties

'get drawing control
Public Property Get Canvas() As PictureBox
  Set Canvas = Picture1
End Property

'standard autoredraw
Public Property Get AutoRedraw() As Boolean
  AutoRedraw = Picture1.AutoRedraw
End Property

Public Property Let AutoRedraw(ByVal vNewValue As Boolean)
  Picture1.AutoRedraw = vNewValue
  PropertyChanged "Picture1.AutoRedraw"
End Property

'set or get warna latar secara serta merta
Public Property Get BackColor() As OLE_COLOR
  BackColor = Picture1.BackColor
End Property

Public Property Let BackColor(ByVal vNewValue As OLE_COLOR)
  Picture1.BackColor = vNewValue
  PropertyChanged "BackColor"
End Property

'warna yang akan diguna untuk melukis grid
Public Property Get GridColor() As OLE_COLOR
  GridColor = grid_color
End Property

Public Property Let GridColor(ByVal vNewValue As OLE_COLOR)
  grid_color = vNewValue
  PropertyChanged "GridColor"
End Property

'warna yang akan diguna untuk melukis garis asalan
Public Property Get OriginColor() As OLE_COLOR
  OriginColor = origin_color
End Property

Public Property Let OriginColor(ByVal vNewValue As OLE_COLOR)
  origin_color = vNewValue
  PropertyChanged "OriginColor"
End Property

'set or get warna untuk garis graf secara serta merta
Public Property Get GraphColor() As OLE_COLOR
  GraphColor = Picture1.ForeColor
End Property

Public Property Let GraphColor(ByVal vNewValue As OLE_COLOR)
  Picture1.ForeColor = vNewValue
  PropertyChanged "GraphColor"
End Property

'style yang akan diguna untuk melukis grid
Public Property Get GridStyle() As DrawStyleConstants
  GridStyle = grid_style
End Property

Public Property Let GridStyle(ByVal vNewValue As DrawStyleConstants)
  grid_style = vNewValue
  PropertyChanged "GridStyle"
End Property

'style yang akan diguna untuk melukis garis asalan
Public Property Get OriginStyle() As DrawStyleConstants
  OriginStyle = origin_style
End Property

Public Property Let OriginStyle(ByVal vNewValue As DrawStyleConstants)
  origin_style = vNewValue
  PropertyChanged "OriginStyle"
End Property

'set or get style untuk garis graf secara serta merta
Public Property Get GraphStyle() As DrawStyleConstants
  GraphStyle = Picture1.DrawStyle
End Property

Public Property Let GraphStyle(ByVal vNewValue As DrawStyleConstants)
  Picture1.DrawStyle = vNewValue
  PropertyChanged "GraphStyle"
End Property

'lebar yang akan diguna untuk melukis grid
Public Property Get GridWidth() As Long
  GridWidth = grid_width
End Property

Public Property Let GridWidth(ByVal vNewValue As Long)
  If vNewValue < 1 Then Exit Property
  grid_width = vNewValue
  PropertyChanged "GridWidth"
End Property

'lebar yang akan diguna untuk melukis garis asalan
Public Property Get OriginWidth() As Long
  OriginWidth = origin_width
End Property

Public Property Let OriginWidth(ByVal vNewValue As Long)
  If vNewValue < 1 Then Exit Property
  origin_width = vNewValue
  PropertyChanged "OriginWidth"
End Property

'set or get lebar untuk garis graf secara serta merta
Public Property Get GraphWidth() As Long
  GraphWidth = Picture1.DrawWidth
End Property

Public Property Let GraphWidth(ByVal vNewValue As Long)
  If vNewValue < 1 Then Exit Property
  Picture1.DrawWidth = vNewValue
  PropertyChanged "GraphWidth"
End Property

'drawmode 'added on 10 may 2008
Public Property Get DrawMode() As DrawModeConstants
  DrawMode = Picture1.DrawMode
End Property

Public Property Let DrawMode(ByVal vNewValue As DrawModeConstants)
  Picture1.DrawMode = vNewValue
  PropertyChanged "DrawMode"
End Property

'/////////////////////////////////
'// limit and grid properties

Public Property Get XMin() As Double
  XMin = min_x
End Property

Public Property Let XMin(ByVal vNewValue As Double)
  If vNewValue >= max_x Then Exit Property
  min_x = vNewValue
  ScalingControl
  PropertyChanged "XMin"
End Property

Public Property Get XMax() As Double
  XMax = max_x
End Property

Public Property Let XMax(ByVal vNewValue As Double)
  If vNewValue <= min_x Then Exit Property
  max_x = vNewValue
  ScalingControl
  PropertyChanged "XMax"
End Property

Public Property Get YMin() As Double
  YMin = min_y
End Property

Public Property Let YMin(ByVal vNewValue As Double)
  If vNewValue >= max_y Then Exit Property
  min_y = vNewValue
  ScalingControl
  PropertyChanged "YMin"
End Property

Public Property Get YMax() As Double
  YMax = max_y
End Property

Public Property Let YMax(ByVal vNewValue As Double)
  If vNewValue <= min_y Then Exit Property
  max_y = vNewValue
  ScalingControl
  PropertyChanged "YMax"
End Property

Public Property Get XScale() As Double
  XScale = scale_x
End Property

Public Property Let XScale(ByVal vNewValue As Double)
  If vNewValue <= 0 Then Exit Property
  scale_x = vNewValue
  PropertyChanged "XScale"
End Property

Public Property Get YScale() As Double
  YScale = scale_y
End Property

Public Property Let YScale(ByVal vNewValue As Double)
  If vNewValue <= 0 Then Exit Property
  scale_y = vNewValue
  PropertyChanged "YScale"
End Property

Public Property Get XGrid() As Integer
  XGrid = div_x
End Property

Public Property Let XGrid(ByVal vNewValue As Integer)
  If vNewValue <= 0 Then Exit Property
  div_x = vNewValue
  PropertyChanged "XGrid"
End Property

Public Property Get YGrid() As Integer
  YGrid = div_y
End Property

Public Property Let YGrid(ByVal vNewValue As Integer)
  If vNewValue <= 0 Then Exit Property
  div_y = vNewValue
  PropertyChanged "YGrid"
End Property

'//////////////
'// events

Private Sub Picture1_Resize()
  RaiseEvent Resize
End Sub

Private Sub Picture1_Paint()
  RaiseEvent Paint
End Sub

Private Sub Picture1_MouseDown(Button As Integer, Shift As Integer, x As Single, y As Single)
  RaiseEvent MouseDown(Button, Shift, x, y)
End Sub

Private Sub Picture1_MouseMove(Button As Integer, Shift As Integer, x As Single, y As Single)
  RaiseEvent MouseMove(Button, Shift, x, y)
End Sub

Private Sub Picture1_MouseUp(Button As Integer, Shift As Integer, x As Single, y As Single)
  RaiseEvent MouseUp(Button, Shift, x, y)
End Sub

'////////////////////////////////
'// scaling member functions

'ubah graf control scaling
Private Sub ScalingControl()

  'scaling graph width
  If min_x <> max_x Then
    Picture1.ScaleWidth = max_x - min_x
    Picture1.ScaleLeft = min_x
  End If

  'scaling graph height
  If min_y <> max_y Then
    Picture1.ScaleHeight = -(max_y - min_y)
    Picture1.ScaleTop = max_y
  End If

End Sub

'get or set graph limit
Public Sub Limit(minx As Double, miny As Double, maxx As Double, maxy As Double, Optional retrieve As Boolean)

  'setting graph limit
  If Not retrieve Then

    If minx < maxx Then
      min_x = minx
      max_x = maxx
    End If

    If miny < maxy Then
      min_y = miny
      max_y = maxy
    End If

    ScalingControl

  'retrieving graph limit
  Else
    minx = min_x
    maxx = max_x
    miny = min_y
    maxy = max_y
  End If

End Sub

'set scaling/distance between grid line
Public Sub Grid(XScale As Double, YScale As Double, xdivision As Integer, ydivision As Integer, Optional retrieve As Boolean)

  'setting graph grid scaling
  If Not retrieve Then
    If XScale > 0 Then scale_x = XScale
    If YScale > 0 Then scale_y = YScale
    If xdivision > 0 Then div_x = xdivision
    If ydivision > 0 Then div_y = ydivision

  'retrieving graph grid scaling
  Else
    XScale = scale_x
    YScale = scale_y
    xdivision = div_x
    ydivision = div_y
  End If

End Sub

'///////////////////////////////////
'// drawing utilities functions

Public Sub DrawDotLine(x1 As Double, y1 As Double, x2 As Double, y2 As Double)
  Const jarakDot As Double = 3
  Dim skipDot As Double
  Dim cp As Double

  'choose line type
  If y1 = y2 Then GoTo DrawHorizontalLine
  If x1 = x2 Then GoTo DrawVerticalLine
  GoTo DrawSlopeLine

Exit Sub

'draw horizontal dot line
DrawHorizontalLine:

  'get picture width in pixel and skipDot in scale mode
  skipDot = ScaleX(Picture1.Width, ScaleMode, vbPixels)
  skipDot = jarakDot * Abs(Picture1.ScaleWidth) / skipDot
  If x1 > x2 Then skipDot = -skipDot

  'draw horizontal dot line
  For cp = x1 To x2 Step skipDot
    Picture1.PSet (cp, y1)
  Next cp

  'draw last dot
  Picture1.PSet (x2, y1)
Exit Sub

'draw vertical dot line
DrawVerticalLine:

  'get picture height in pixel and skipDot in scale mode
  skipDot = ScaleY(Picture1.Height, ScaleMode, vbPixels)
  skipDot = jarakDot * Abs(Picture1.ScaleHeight) / skipDot
  If y1 > y2 Then skipDot = -skipDot

  'draw vertical dot line
  For cp = y1 To y2 Step skipDot
    Picture1.PSet (x1, cp)
  Next cp

  'draw last dot
  Picture1.PSet (x1, y2)
Exit Sub

'draw sloped dot line
DrawSlopeLine:
  Dim px As Double
  Dim py As Double
  Dim xRange As Double
  Dim yRange As Double
  Dim length As Double
  Dim sudut As Double
  Dim dx As Double
  Dim dy As Double
  Dim i As Long

  'starting dot
  px = x1
  py = y1

  'find length of line
  xRange = x2 - x1
  yRange = y2 - y1
  length = Sqr(xRange * xRange + yRange * yRange)

  'find angle of line
  sudut = Atn(yRange / xRange)

  'find skipDot
  dx = ScaleX(Picture1.Width, ScaleMode, vbPixels)
  dy = ScaleY(Picture1.Height, ScaleMode, vbPixels)
  If dx > dy Then skipDot = jarakDot * Abs(Picture1.ScaleWidth) / dx
  If dy >= dx Then skipDot = jarakDot * Abs(Picture1.ScaleHeight) / dy

  'find dot increment
  dx = Abs(skipDot * Cos(sudut))
  dy = Abs(skipDot * Sin(sudut))
  If xRange < 0 Then dx = -dx
  If yRange < 0 Then dy = -dy
  length = length / skipDot

  'drawing sloped dot line
  For i = 1 To length
    Picture1.PSet (px, py)
    px = px + dx
    py = py + dy
  Next i

  'draw last dot
  Picture1.PSet (x2, y2)
End Sub

Public Sub DrawGridLine(Optional skipOrigin As Boolean)
  Dim prevDrawColor As OLE_COLOR
  Dim prevDrawStyle As DrawStyleConstants
  Dim prevDrawWidth As Long

  'save graph drawing parameters
  prevDrawColor = GraphColor
  prevDrawStyle = GraphStyle
  prevDrawWidth = GraphWidth

  'set drawing mode for grid
  GraphColor = grid_color
  GraphStyle = grid_style
  GraphWidth = grid_width

  'draw predefined style grid
  GoSub drawGrid

  If Not skipOrigin Then

    'set drawing mode for origin
    GraphColor = origin_color
    GraphStyle = origin_style
    GraphWidth = origin_width

    'draw predefined style origin
    GoSub drawOrigin
  End If

  'restore graph drawing parameters
  GraphColor = prevDrawColor
  GraphStyle = prevDrawStyle
  GraphWidth = prevDrawWidth
Exit Sub

drawGrid:
  Dim grid_scale As Double
  Dim grid_end As Double
  Dim cp As Double
  Dim i As Long

  'find scaling for small vertical grid division
  grid_scale = scale_x / div_x
  If grid_scale <= 0 Then
    grid_scale = ScaleX(Picture1.Width, ScaleMode, vbPixels)
    grid_scale = Abs(Picture1.ScaleWidth) / grid_scale
  End If

  'drawing positive vertical division
  grid_end = max_x
  GoSub DrawLoopVerticalGrid

  'drawing negative vertical division
  grid_scale = -grid_scale
  grid_end = min_x
  GoSub DrawLoopVerticalGrid

  'find scaling for small horizontal grid division
  grid_scale = scale_y / div_y
  If grid_scale <= 0 Then
    grid_scale = ScaleY(Picture1.Height, ScaleMode, vbPixels)
    grid_scale = Abs(Picture1.ScaleHeight) / grid_scale
  End If

  'drawing positive horizontal division
  grid_end = max_y
  GoSub DrawLoopHorizontalGrid

  'drawing negative horizontal division
  grid_scale = -grid_scale
  grid_end = min_y
  GoSub DrawLoopHorizontalGrid
Return

DrawLoopVerticalGrid:
  i = 0
  For cp = grid_scale To grid_end Step grid_scale
    i = i + 1
    If i < div_x Then 'draw small grid
      DrawDotLine cp, min_y, cp, max_y
    Else 'draw main grid
      i = 0
      Picture1.Line (cp, min_y)-(cp, max_y)
    End If
  Next cp
Return

DrawLoopHorizontalGrid:
  i = 0
  For cp = grid_scale To grid_end Step grid_scale
    i = i + 1
    If i < div_y Then 'draw small grid
      DrawDotLine min_x, cp, max_x, cp
    Else 'draw main grid
      i = 0
      Picture1.Line (min_x, cp)-(max_x, cp)
    End If
  Next cp
Return

drawOrigin:

  'origin line
  Picture1.Line (0, min_y)-(0, max_y)
  Picture1.Line (min_x, 0)-(max_x, 0)
Return
End Sub

'///////////////////////////////
'// graph drawing functions

'clear all drawing
Public Sub Clear()
  Picture1.Cls
End Sub

'clearing graph and updating scaling
Public Sub Refresh()
  ScalingControl
  Clear
End Sub

'start new graph line
Public Sub Start()
  graph_start = True
End Sub

'draw and connect graph data and line
Public Sub Connect(x As Double, y As Double)

  'plotting 1st point
  If graph_start Then
    graph_start = False
    Picture1.PSet (x, y)

  'plotting next point
  Else
    Picture1.Line -(x, y)
  End If

End Sub
